_A=False
import logging,os,re,threading,traceback
from contextlib import contextmanager
from typing import List
from localstack import constants
from localstack.constants import MAVEN_REPO_URL
from localstack.packages import InstallTarget,PackageException
from localstack.services.install import ARTIFACTS_REPO,JAR_URLS,Installer,InstallerRepository,add_file_to_jar,update_jar_manifest
from localstack.services.stepfunctions.packages import SFN_PATCH_URL_PREFIX
from localstack.utils.files import file_exists_not_empty
from localstack.utils.http import download
from localstack.utils.time import now
from localstack_ext.bootstrap.installer import mariadb_installer,mosquitto_installer,postgres_installer,timescaledb_installer
from localstack_ext.bootstrap.licensing import api_key_configured,is_enterprise,prepare_environment
LOG=logging.getLogger(__name__)
H2_DOWNLOAD_URL='http://www.h2database.com/h2-2019-10-14.zip'
SSL_CERT_URL=f"{ARTIFACTS_REPO}/raw/master/local-certs/server.key"
SSL_CERT_URL_FALLBACK='{api_endpoint}/proxy/localstack.cert.key'
POSTGRES_LIB_FOLDER='/usr/lib/postgresql/11/lib'
SFN_PATCH_PRO_CLASS1='cloud/localstack/PersistenceAspect.class'
SFN_PATCH_PRO_CLASS2='cloud/localstack/PersistenceContext.class'
SFN_PATCH_PRO_CLASS3='cloud/localstack/PersistenceState.class'
SFN_PATCH_PRO_CLASS4='cloud/localstack/PersistenceRegionState.class'
SFN_PATCH_PRO_FILE_METAINF='META-INF/aop-pro.xml'
URL_KRYO=f"{MAVEN_REPO_URL}/com/esotericsoftware/kryo/5.2.0/kryo-5.2.0.jar"
URL_OBJENESIS=f"{MAVEN_REPO_URL}/org/objenesis/objenesis/3.2/objenesis-3.2.jar"
URL_MINLOG=f"{MAVEN_REPO_URL}/com/esotericsoftware/minlog/1.3.1/minlog-1.3.1.jar"
URL_REFLECTASM=f"{MAVEN_REPO_URL}/com/esotericsoftware/reflectasm/1.11.9/reflectasm-1.11.9.jar"
PRO_JAR_URLS=[URL_KRYO,URL_OBJENESIS,URL_MINLOG,URL_REFLECTASM]
INSTALL_LOCK=threading.RLock()
@contextmanager
def log_package_exception():
	try:yield
	except PackageException:LOG.exception('Package installation failed.')
def install_libs():
	from localstack_ext.services.elasticache.packages import redis_package as A;from localstack_ext.services.iot.packages import iot_rule_engine_package as B;B.install(target=InstallTarget.STATIC_LIBS);postgres_installer.install(raise_on_error=_A);timescaledb_installer.install(raise_on_error=_A)
	with log_package_exception():A.install()
	mosquitto_installer.install(raise_on_error=_A);install_stepfunctions()
def install_stepfunctions():
	G='StepFunctionsLocal.jar';from localstack.services.stepfunctions.packages import stepfunctions_local_package as H;B=H.get_installer();A=B.get_installed_dir();C=B.get_executable_path()
	if not C:LOG.warning('StepFunctions JAR not available - skipping installation');return
	I=[SFN_PATCH_PRO_CLASS1,SFN_PATCH_PRO_CLASS2,SFN_PATCH_PRO_CLASS3,SFN_PATCH_PRO_CLASS4,SFN_PATCH_PRO_FILE_METAINF]
	for D in I:J=f"{SFN_PATCH_URL_PREFIX}/{D}";add_file_to_jar(D,J,target_jar=C)
	update_jar_manifest(G,A,re.compile('Main-Class: com\\.amazonaws.+'),'Main-Class: cloud.localstack.StepFunctionsStarter');K=' '.join([os.path.basename(A)for A in[*(PRO_JAR_URLS),*(JAR_URLS)]]);update_jar_manifest(G,A,re.compile('Class-Path: .+ \\. '),f"Class-Path: {K} . ")
	for E in PRO_JAR_URLS:
		F=os.path.join(A,os.path.basename(E))
		if not file_exists_not_empty(F):download(E,F)
def setup_ssl_cert():
	from localstack.services import generic_proxy as C;A=C.get_cert_pem_file_path()
	if os.path.exists(A):
		if is_enterprise():LOG.debug('Avoiding to update SSL certificate.');return
		D=6*60*60;E=os.path.getmtime(A)
		if E>now()-D:LOG.debug('Using cached SSL certificate (less than 6hrs since last update).');return
	LOG.debug('Attempting to download local SSL certificate file');F=3;G=5
	try:return download_github_artifact(SSL_CERT_URL,A,timeout=F)
	except Exception:
		B=SSL_CERT_URL_FALLBACK.format(api_endpoint=constants.API_ENDPOINT)
		try:return download(B,A,timeout=G)
		except Exception as H:LOG.info('Unable to download local test SSL certificate from %s to %s (using self-signed cert as fallback): %s',B,A,H);raise
def download_github_artifact(url,target_file,timeout=None):
	B=target_file;A=url
	def C(url,print_error=_A):
		try:download(url,B,timeout=timeout);return True
		except Exception as A:
			if print_error:LOG.info('Unable to download Github artifact from from %s to %s: %s %s'%(url,B,A,traceback.format_exc()))
	D=C(A)
	if not D:A=A.replace('https://github.com','https://cdn.jsdelivr.net/gh');A=A.replace('/raw/master/','@master/');C(A,True)
def install_azure():from localstack_ext.services.azure import api_specs as A;A.download_api_specs()
def install_pysiddhi():from localstack_ext.services.kinesisanalytics import query_utils as A;A.setup_siddhi()
class ExtInstallerRepository(InstallerRepository):
	name='ext'
	def should_load(A):return api_key_configured()
	def load(A,*B,**C):
		LOG.debug('Preparing Pro environment for LocalStack Package Manager.')
		with prepare_environment():LOG.debug('Pro environment has successfully been prepared.')
	def get_installer(K):from localstack_ext.packages.postgres import PostgresqlPackage as A;from localstack_ext.services.ecr.packages import registry_package as B;from localstack_ext.services.eks.packages import k3d_package as C;from localstack_ext.services.elasticache.packages import redis_package as D;from localstack_ext.services.iot.packages import iot_rule_engine_package as E;from localstack_ext.services.kafka.packages import kafka_package as F;from localstack_ext.services.mq.packages import amazon_mq_package as G;from localstack_ext.services.neptune.packages import neo4j_package as H,tinkerpop_package as I;from localstack_ext.services.qldb.packages import partiql_package as J;return[('iot-rule-engine',E),('postgres',A()),('timescaledb',timescaledb_installer.install),('redis',D.install),('mqtt',mosquitto_installer.install),('azure',install_azure),('ecr',B),('mysql',mariadb_installer.install),('pysiddhi',install_pysiddhi),('neo4j',H),('tinkerpop',I),('qldb',J),('k3d',C),('stepfunctions',install_stepfunctions),('amazon-mq',G),('kafka',F)]